package id

import (
	"fmt"
)

const (
	Space        = "space"
	SpacesPrefix = "spaces"
)

var _ Descriptor = &SpaceId{}

type SpaceId struct {
	SpaceID string `json:"space_id,omitempty" bson:"space_id,omitempty"`
}

func (id *SpaceId) New() Descriptor {
	return &SpaceId{}
}

func (id *SpaceId) Type() string { return Space }

func (id *SpaceId) String() string {
	return Join(SpacesPrefix, id.SpaceID)
}

func (id *SpaceId) FromParts(parts []string) error {
	if len(parts) != 2 || parts[0] != SpacesPrefix {
		return ErrInvalidID
	}

	id.SpaceID = parts[1]
	return nil
}

func (id *SpaceId) Map() map[string]any {
	return map[string]any{
		"space_id": id.SpaceID,
		"type":     id.Type(),
	}
}

func (id *SpaceId) FromMap(m map[string]any) error {
	id.SpaceID, _ = m["space_id"].(string)
	if id.SpaceID == "" {
		return fmt.Errorf("%w: SpaceId required", ErrInvalidID)
	}
	return nil
}

func (id *SpaceId) Validate() error {
	if id.SpaceID == "" {
		return fmt.Errorf("%w: SpaceId required", ErrInvalidID)
	}
	return nil
}

func NewSpaceId(id string) *ObjectId {
	return &ObjectId{Descriptor: &SpaceId{SpaceID: id}}
}
